/*
 * lut.c --  Look-up-table functions for pseudocoloring data.
 */

#include "muse.h"

#define LUT_NO_FIND -1



FILE_SPEC       file =
{
'\0', '\0', '\0'};

#if XVT_CC_PROTO
MUSE_API lut_construct(FILE * file, PALETTE * pal, LUT ** pointer)
#else
MUSE_API lut_construct(file, pal, pointer)
    FILE           *file;
    PALETTE        *pal;
    LUT           **pointer;
#endif
{
    LUT            *lut;
    short           i;
    size_t          items_read = 0;
    unsigned char   lutrec[SZ_LUT];
    long            temp = SZ_LUT;
    char            string[80];

    lut = (LUT *) muse_malloc(sizeof(LUT), "lut");
    if (lut == (LUT *) NULL)
	return (STAT_MEM_ALLOC_ERR);
    *pointer = lut;
    memset(lut, '\0', sizeof(LUT));
    lut->magic = MAGIC_LUT;

#if XVTOS == DOSOS
    if (temp < sizeof(LUT))
	printf("SZ_LUT must be at least %u bytes!", sizeof(RASTER));
#endif

    if (file)
    {
	sprintf(string, "read raster lut: %ld\n", ftell(file));
	/*muse_log(string);*/
	items_read = fread((void *) lutrec, sizeof(lutrec), (size_t) 1, file);
	decode_lut(lutrec, lut);
	if (items_read != 1)
	{
	    printf("Unable to read in the lut");
	    return (STAT_FILE_READ_ERR);
	}
	if (lut->magic != MAGIC_LUT)
	{
	    printf("The lut data read in is incorrect.");
	    return (STAT_DATA_ERR);
	}
	*pointer = lut;
	return STAT_SUCCESS;
    }
    /*    xvt_fsys_get_default_dir(&lut->lut_file.dir);*/
    lut->lut_file.type[0] = '\0';
    lut->lut_file.name[0] = '\0';

    lut->lut_desc[0].num_hues = 6;
    lut->lut_desc[0].num_shades = 3;
    lut->lut_desc[0].data_range[0] = 0.0;
    lut->lut_desc[0].data_range[1] = 3000.0;
    lut->lut_desc[0].hue_range[0] = 480.0;
    lut->lut_desc[0].hue_range[1] = 300.0;
    lut->lut_desc[0].shade_range[0].sat = 0.7;
    lut->lut_desc[0].shade_range[0].val = 1.0;
    lut->lut_desc[0].shade_range[1].sat = 1.0;
    lut->lut_desc[0].shade_range[1].val = 1.0;
    lut->lut_desc[0].shade_range[2].sat = 1.0;
    lut->lut_desc[0].shade_range[2].val = 0.4;

    lut->lut_desc[1].num_hues = 6;
    lut->lut_desc[1].num_shades = 3;
    lut->lut_desc[1].data_range[0] = -6000.0;
    lut->lut_desc[1].data_range[1] = 0.0;
    lut->lut_desc[1].hue_range[0] = 270.0;
    lut->lut_desc[1].hue_range[1] = 180.0;
    lut->lut_desc[1].shade_range[0].sat = 0.7;
    lut->lut_desc[1].shade_range[0].val = 1.0;
    lut->lut_desc[1].shade_range[1].sat = 1.0;
    lut->lut_desc[1].shade_range[1].val = 1.0;
    lut->lut_desc[1].shade_range[2].sat = 1.0;
    lut->lut_desc[1].shade_range[2].val = 0.3;

    for (i = 2; i < MAX_LUT; i++)
    {
	lut->lut_desc[i].num_hues = 0;
	lut->lut_desc[i].num_shades = 0;
	lut->lut_desc[i].data_range[0] = 0.0;
	lut->lut_desc[i].data_range[1] = 0.0;
	lut->lut_desc[i].hue_range[0] = 0.0;
	lut->lut_desc[i].hue_range[1] = 0.0;
	lut->lut_desc[i].shade_range[0].sat = 0.0;
	lut->lut_desc[i].shade_range[0].val = 0.0;
	lut->lut_desc[i].shade_range[1].sat = 0.0;
	lut->lut_desc[i].shade_range[1].val = 0.0;
	lut->lut_desc[i].shade_range[2].sat = 0.0;
	lut->lut_desc[i].shade_range[2].val = 0.0;
    }
    lut2pal(lut, ALL_PURPOSE, pal);

    return STAT_SUCCESS;
}				/* end lut_construct */

#if XVT_CC_PROTO
MUSE_API lut_destruct(
	     FILE * file,
	     BOOLEAN destruct,
	     LUT ** lut)
#else
MUSE_API lut_destruct(file, destruct, lut)
    FILE           *file;
    BOOLEAN         destruct;
    LUT           **lut;
#endif
{
    size_t          items_written = 0;
    unsigned char   lutrec[SZ_LUT];
    char            string[80];

    if (*lut == (LUT *) NULL)
	return STAT_SUCCESS;

    if (file)
    {
	encode_lut(lutrec, *lut);
	sprintf(string, "writ lut: %ld\n", ftell(file));
	/*muse_log(string);*/
	items_written = fwrite((void *) lutrec, sizeof(lutrec), (size_t) 1, file);
	if (items_written != 1)
	{
	    printf("Unable to write the lut");
	    return (STAT_FILE_WRITE_ERR);
	}
    }
    if (destruct)
    {

	muse_free(*lut, "lut");
	*lut = (LUT *) NULL;
    }
    return (STAT_SUCCESS);
}


#if XVT_CC_PROTO
ERRSTATUS
lut_lookup(LUT * lut, GFLOAT data, GFLOAT delta,
	   GFLOAT shade_threshold, GFLOAT precision, USHORT * pal_index)
#else
ERRSTATUS
lut_lookup(lut, data, delta, shade_threshold,
	   precision, pal_index)
    LUT            *lut;
    GFLOAT          data;
    GFLOAT          delta;
    GFLOAT          shade_threshold;
    GFLOAT          precision;
    USHORT         *pal_index;
#endif
{
    int             i, ihue, ishade, offst = 0;
    short           xmid;
    GFLOAT          aniota = 0.0001;	/* used to avoid divide by zero */
    USHORT          lut_map_index;

    static GFLOAT   last_data, last_delta;
    static short    last_index;
    NOREF(precision);
    for (i = 0; i < MAX_LUT; i++)
    {
	if (data >= lut->lut_desc[i].data_range[0] &&
	    data <= lut->lut_desc[i].data_range[1])
	    break;
	else
	    offst = offst + lut->lut_desc[i].num_shades *
		      lut->lut_desc[i].num_hues;
    }
    if (i == MAX_LUT)
	return LUT_NO_FIND;


    ihue = ((data - lut->lut_desc[i].data_range[0]) /
	    (lut->lut_desc[i].data_range[1] -
	     lut->lut_desc[i].data_range[0])) *
	      lut->lut_desc[i].num_hues;


    if (lut->lut_desc[i].num_shades == 1)
	lut_map_index = ihue + offst;
    else if (lut->lut_desc[i].num_shades == 2)
    {
	if (delta < 0.0)
	    lut_map_index = (ihue * lut->lut_desc[i].num_shades) + offst;
	else
	    lut_map_index = (ihue * lut->lut_desc[i].num_shades + 1) + offst;
    }
    else
    {
	xmid = lut->lut_desc[i].num_shades / 2;
	if (shade_threshold > aniota)
	    ishade = xmid + (delta / shade_threshold);
	else
	    ishade = xmid + (delta / aniota);
	if (ishade < 0)
	    ishade = 0;
	if (ishade > lut->lut_desc[i].num_shades - 1)
	    ishade = lut->lut_desc[i].num_shades - 1;
	lut_map_index = ((ihue * lut->lut_desc[i].num_shades)
			 + ishade) + offst;
    }
    last_data = data;
    last_delta = delta;
    *pal_index = lut->map[lut_map_index];
    last_index = *pal_index;
    return STAT_SUCCESS;
}

