/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * MapElement -- a geographic coordinate list or text element
 */

#ifndef mapelt_h
#define mapelt_h

extern "C" {
#include "projects.h"
}

#include <Map/mapfeature.h>


class MapElement : public MapFeature {
public:
  MapElement(int id, MapFeatureType, int, double* x, double* y, void* = nil);
  MapElement(int id, char*, void* = nil);
  MapElement();
  virtual ~MapElement();

  void* graphic() const;
  int points(const double*& x, const double*& y) const;
  void points(double*& x, double*& y, int&) const;
  void points(PJ*, double*& x, double*& y, int&) const;
  int num_points() const;
  void point(int, double&, double&) const;
  const char* text() const;

  void graphic(void*);
  void points(int, double*, double*);
  void text(char*);

  virtual const char* describe() const;
protected:
  void init_points(int, double*, double*);
  int _en;
  double* _ex;
  double* _ey;
  void* _gr;
  char* _textstr;
  int _textlen;
};

class GeoPoint : public MapElement {
public:
  GeoPoint(int id, double x, double y, void* = nil);
  virtual const char* describe() const;

    virtual int npts() const { return 0; }
    virtual float * xpoints() const { return nil; }
    virtual float * ypoints() const { return nil; }
    virtual float * zpoints() const { return nil; }
    virtual void origin(float &, float &, float &) const {}
    virtual float xorigin() const { return 0.0; }
    virtual float yorigin() const { return 0.0; }
    virtual float zorigin() const { return 0.0; }
    virtual float xextent() const { return 0.0; }
    virtual float yextent() const { return 0.0; }
    virtual float zextent() const { return 0.0; }
protected:
  static char _desc[256];
};

#endif
