/* This code comes from the MAGIC library at http://www.cs.unc.edu/~eberly/ */

#ifndef GEOMDIST_H
#define GEOMDIST_H

typedef struct
{
	double x, y, z;
}
Point3;

typedef struct
{
	// Line points are L(t) = b+t*m for 0 <= t <= 1

	Point3 b, m;
}
Line3;

typedef struct
{
	// Triangle points are tri(s,t) = s*e0+t*e1+b where 0 <= s <= 1,
	// 0 <= t <= 1, and 0 <= s+t <= 1.

	Point3 b, e0, e1;
}
Triangle3;

typedef struct
{
	// Rectoid points are rect(s,t) = s*e0+t*e1+b where 0 <= s <= 1
	// and 0 <= t <= 1.  Could have called this a parallelogram, but
	// I do not like typing long words...

	Point3 b, e0, e1;
}
Rectoid3;


double MinPointLine (const Point3& p, const Line3& line,
					 double& t);

double MinLineLine (const Line3& line0, const Line3& line1,
					double& s, double& t);

double MinPointTriangle (const Point3& p, const Triangle3& tri,
						 double& s, double& t);

double MinLineTriangle (const Line3& line, const Triangle3& tri,
						double& r, double& s, double& t);

double MinTriangleTriangle (const Triangle3& tri0, const Triangle3& tri1,
							double& s, double& t, double& u, double& v);

double MinPointRectoid (const Point3& p, const Rectoid3& rect,
						double& s, double& t);

double MinLineRectoid (const Line3& line, const Rectoid3& rect,
					   double& r, double& s, double& t);

double MinTriangleRectoid (const Triangle3& tri, const Rectoid3& rect,
						   double& s, double& t, double& u, double& v);

double MinRectoidRectoid (const Rectoid3& rect0, const Rectoid3& rect1,
						  double& s, double& t, double& u, double& v);

#endif
